@@ -27,9 +27,9 @@
 #include <kconfig.h>
 #include <kcursor.h>
 
-KGameWidget::KGameWidget(QWidget *parent,KPuzzleApp* app) : QWidget(parent)
+KGameWidget::KGameWidget(QWidget *parent) : QWidget(parent)
 {
-	_app = app;
+	_app = NULL;
 	setMouseTracking(true);
         setCursor(KCursor::blankCursor());
 	_piecePos = QPoint(-1,-1);
@@ -151,7 +151,8 @@ void KGameWidget::mousePressEvent(QMouseEvent* event)
 
 		_app->updateAll();
 		break;
-	}
+            default: ;
+        }
 }
 
 void KGameWidget::mouseReleaseEvent(QMouseEvent* event)
