@@ -35,7 +35,8 @@ class KGameWidget : public QWidget
 {
 	Q_OBJECT
  public:
-	KGameWidget(QWidget *parent,KPuzzleApp* app);
+	KGameWidget(QWidget *parent);
+        void setApp(KPuzzleApp* app) { _app = app; }
 	virtual ~KGameWidget();
 // Initialize some things that cannot be done in the constructor
 	void initialize(QSize picSize);
