@@ -21,6 +21,7 @@
 #include "kpuzzleview.h"
 #include "kpuzzleapp.h"
 #include "prefs.h"
+#include "highscore.h"
 
 #include <qpainter.h>
 #include <qpopupmenu.h>
@@ -30,26 +31,22 @@
 
 #include <kglobal.h>
 #include <klocale.h>
+#include <kcrash.h>
 #include <kmenubar.h>
 #include <kaccel.h>
-//#include <kio/netaccess.h>
 #include <kfiledialog.h>
 #include <kconfig.h>
-//#include <kurl.h>
-//#include <kurlrequesterdlg.h>
-
-//#include <kedittoolbar.h>
 
 #include <kstdaccel.h>
 #include <kaction.h>
 #include <kstdaction.h>
 #include <kpopupmenu.h>
 
-#ifdef DEBUG
+/*#ifdef DEBUG
 #include <sys/types.h>
 #include <signal.h>
 #include <unistd.h>
-#endif
+#endif*/
 
 KPuzzle::KPuzzle() : KMainWindow( 0, "KPuzzle" )
 {
@@ -74,19 +71,8 @@ KPuzzle::KPuzzle() : KMainWindow( 0, "KPuzzle" )
                 this,   SLOT(changeStatusbar(const QString&)));
 
 #ifdef DEBUG
-/*        struct sigaction* sigdefault;
-        sigdefault->sa_handler = SIG_DFL;
-        sigdefault->sa_flags = 0;
-        sigaction(SIGSEGV,sigdefault,NULL);
-        sigaction(SIGABRT,sigdefault,NULL);*/
-        signal(SIGSEGV,SIG_DFL);
-        signal(SIGABRT,SIG_DFL);
+        KCrash::setCrashHandler(NULL);
 #endif
-
-//        maskCache = new QCache<QBitmap>;
-//        blackPixmap = new QPixmap;
-        KGlobal::locale()->setLanguage("de");
-
 }
 
 KPuzzle::~KPuzzle()
@@ -99,58 +85,64 @@ void KPuzzle::createGUI()
 
         KAction* temp;
 
-        temp = KStdAction::openNew(m_view->game(), SLOT(slotNewGame()), this);
+        temp = KStdAction::openNew(m_view, SLOT(slotNewGame()), this);
         temp->plug(fileMenu);
         temp->plug(toolBar());
 
-        temp = KStdAction::open(m_view->game(), SLOT(slotOpenGame()), this);
+        temp = KStdAction::open(m_view, SLOT(slotOpenGame()), this);
         temp->plug(fileMenu);
         temp->plug(toolBar());
 
-        m_view->game()->actSave() = KStdAction::save(m_view->game(), SLOT(slotSaveGame()), this);
-        m_view->game()->actSave()->plug(fileMenu);
-        m_view->game()->actSave()->plug(toolBar());
+//        m_view->actSave() = KStdAction::save(m_view, SLOT(slotSaveGame()), this);
+        m_view->actSave() = KStdAction::save(0,0,this);
+        m_view->actSave()->plug(fileMenu);
+        m_view->actSave()->plug(toolBar());
 
-        m_view->game()->actSaveAs() = KStdAction::saveAs(m_view->game(), SLOT(slotSaveGameAs()), this);
-        m_view->game()->actSaveAs()->plug(fileMenu);
+//        m_view->actSaveAs() = KStdAction::saveAs(m_view->game(), SLOT(slotSaveGameAs()), this);
+        m_view->actSaveAs() = KStdAction::saveAs(0,0, this);
+        m_view->actSaveAs()->plug(fileMenu);
 
-        m_view->game()->actTerminate() = new KAction(i18n("&Terminate game"),QIconSet(QPixmap(PICFILE("stop.xpm"))),0,m_view->game(),
-                                                     SLOT(slotStopGame()),this);
-        m_view->game()->actTerminate()->plug(fileMenu);
+//        m_view->actTerminate() = new KAction(i18n("&Terminate game"),QIconSet(QPixmap(PICFILE("stop.xpm"))),0,m_view->game(),
+//                                               SLOT(slotStopGame()),this);
+        m_view->actTerminate() = new KAction(i18n("&Terminate game"),QIconSet(QPixmap(PICFILE("stop.xpm"))),0,0,
+                                             0,this);
+        m_view->actTerminate()->plug(fileMenu);
 
         (new KActionSeparator(this))->plug(fileMenu);
         (new KActionSeparator(this))->plug(toolBar());
 
         KStdAction::preferences(this, SLOT(slotSetOptions()), this)->plug(fileMenu);
 
-        temp = new KAction(i18n("&Highscores"),0,m_view->game(),
-                           SLOT(slotHighscores()),this);
-        temp->plug(fileMenu);
+//         m_view->actHighscores() = new KAction(i18n("&Highscores"),0,m_view->game(),
+//                                               SLOT(slotHighscores()),this);
+        m_view->actHighscores() = new KAction(i18n("&Highscores"),0,this,
+                                               SLOT(slotHighscores()),this);
+        m_view->actHighscores()->plug(fileMenu);
 
         KStdAction::quit(kapp, SLOT(quit()), this)->plug(fileMenu);
 
-        menuBar()->insertItem("&File",fileMenu);
+        menuBar()->insertItem(i18n("&File"),fileMenu);
 
         QPopupMenu* gameMenu = new QPopupMenu(menuBar());
 
-        m_view->game()->actPause() = new KAction(i18n("&Pause Game"),QIconSet(QPixmap(PICFILE("pause.xpm"))),0,m_view->game(),
-                                                  SLOT(slotPause()),this);
-        m_view->game()->actPause()->plug(gameMenu);
-        m_view->game()->actPause()->plug(toolBar());
-        debug((const char*) (PICFILE("pause.xpm")));
+        m_view->actPause() = new KAction(i18n("&Pause Game"),QIconSet(QPixmap(PICFILE("pause.xpm"))),0,0,
+                                                  0,this);
+        m_view->actPause()->plug(gameMenu);
+        m_view->actPause()->plug(toolBar());
+
+        m_view->actTerminate()->plug(toolBar());
 
-        m_view->game()->actTerminate()->plug(toolBar());
         (new KActionSeparator(this))->plug(toolBar());
 
-        m_view->game()->actShowLarge() = new KAction(i18n("&Show large image"),QIconSet(QPixmap(PICFILE("large.xpm"))),0,m_view->game(),
-                                                     SLOT(slotShowLarge()),this);
-        m_view->game()->actShowLarge()->plug(gameMenu);
-        m_view->game()->actShowLarge()->plug(toolBar());
+        m_view->actShowLarge() = new KAction(i18n("&Show large image"),QIconSet(QPixmap(PICFILE("large.xpm"))),0,0,
+                                             0,this);
+        m_view->actShowLarge()->plug(gameMenu);
+        m_view->actShowLarge()->plug(toolBar());
 
-        m_view->game()->actShowMainPixmap() = new KAction(i18n("&Show full image"),QIconSet(QPixmap(PICFILE("large-main.xpm"))),
-                                                          0,m_view->game(),SLOT(slotShowMainPixmap()),this);
-        m_view->game()->actShowMainPixmap()->plug(gameMenu);
-        m_view->game()->actShowMainPixmap()->plug(toolBar());
+        m_view->actShowMainPixmap() = new KAction(i18n("&Show full image"),QIconSet(QPixmap(PICFILE("large-main.xpm"))),
+                                                  0,0,0,this);
+        m_view->actShowMainPixmap()->plug(gameMenu);
+        m_view->actShowMainPixmap()->plug(toolBar());
 
 
         menuBar()->insertItem(i18n("&Game"),gameMenu);
@@ -158,7 +150,7 @@ void KPuzzle::createGUI()
         KPopupMenu* hMenu = this->helpMenu(i18n("KPuzzle - A KDE jigsaw puzzle game"),false);
         menuBar()->insertItem(i18n("&Help"),hMenu);
 
-        m_view->game()->enableMenuItems(MI_NONE);
+        m_view->enableMenuItems(MI_NONE);
 }
 
 void KPuzzle::saveProperties(KConfig *config)
@@ -199,8 +191,11 @@ void KPuzzle::slotSetOptions()
 {
 //	bool timerActive = m_view->game()->timerActive();
 //	if (timerActive) m_view->game()->stopTimer();
-        bool gameIsPaused = m_view->game()->paused();
-        m_view->game()->pause(true);
+        bool gameIsPaused;
+        if (m_view->game()) {
+                gameIsPaused = m_view->game()->paused();
+                m_view->game()->pause(true);
+        }
 	CPrefsDialog dlg(this);
 	KConfig* cfg =  kapp->sessionConfig();
 	dlg._usePieceMask = cfg->readBoolEntry("UsePieceMask",SV_USE_PIECE_MASK);
@@ -226,5 +221,19 @@ void KPuzzle::slotSetOptions()
 		cfg->sync();
 	}
 //	if (timerActive) m_view->game()->startTimer();
-        m_view->game()->pause(gameIsPaused);
+        if (m_view->game()) m_view->game()->pause(gameIsPaused);
 }
+
+void KPuzzle::slotHighscores()
+{
+        bool gameIsPaused = false;
+        if (m_view->game()) {
+                gameIsPaused = m_view->game()->paused();
+                m_view->game()->pause(true);
+        }
+ 	CHighscoreDialog hs(this,0);
+	hs.initializeDialog();
+	hs.exec();
+        if (m_view->game()) m_view->game()->pause(gameIsPaused);
+}
+
