@@ -25,6 +25,7 @@
 #include <qbitmap.h>
 //#include <qmessagebox.h>
 //#include <qfiledialog.h>
+#include <qwidget.h>
 #include <qdialog.h>
 #include <qfile.h>
 #include <qdatastream.h>
@@ -244,8 +245,6 @@ KPuzzleApp::KPuzzleApp(KPuzzleView* view)
         _mainPixmap = NULL;
         _currentData = new CurrentData;
         currentData()->_owner = this;
-
-        blackPixmap = new QPixmap;
 }
 
 bool KPuzzleApp::initialize()
@@ -263,15 +262,9 @@ bool KPuzzleApp::initializeNew()
 
         gameData()->_savePixmap = false;
 
-        _mainPixmap = new QPixmap;
 
 	gameFilename() = QString::null;
 
-	_mainPixmap->load(gameData()->_filename);
-	if (_mainPixmap->isNull()) {
-                delete _mainPixmap;
-		return false;
-	}
 
 // 	QPainter pt;
 // 	pt.begin(_gamePixmap);
@@ -295,14 +288,6 @@ bool KPuzzleApp::initializeNew()
 	ASSERT(piecesCount().width() >= 3);
 	ASSERT(piecesCount().height() >= 3);
 
-// Moved to calcPiecesCount
-// 	gameData()->_pieceSize = QSize(pixmapSize().width() / piecesCount().width(),
-//                            pixmapSize().height() / piecesCount().height());
-
-        debug("Resizing blackPixmap to %d---%d",pieceSizeDisp().width(),pieceSizeDisp().height());
-        blackPixmap->resize(pieceSizeDisp());
-        blackPixmap->fill(QColor(black));
-
         currentData()->_pieceList = new QList<CPiece>;
 	currentData()->_pieceList->setAutoDelete(true);
 	minglePieces();
@@ -313,9 +298,6 @@ bool KPuzzleApp::initializeNew()
 	currentData()->_currentPiecePixmap = new QPixmap(pieceSizeDisp());
 	getPiecePixmap();
 
-// 	_gameStatus = (char*) malloc(piecesTotal());
-// 	memset(_gameStatus,0,piecesTotal());
-
 	_picview->updatePixmap(gamePixmap(),gamePixmap()->rect());
 	_fullview->updatePixmap(mainPixmap(),mainPixmap()->rect());
 
@@ -352,27 +334,16 @@ bool KPuzzleApp::initializeLoad()
 {
         if (!initialize()) return false;
 
-//	gameFilename() = QString::null;
-
-// 	_mainPixmap->load(gameData()->_filename);
-// 	if (_mainPixmap->isNull()) {
-// 		return false;
-// 	}
-
         if (!gameData()->_savePixmap) scalePixmap();
-//                        gameData()->_piecesCount += QSize(1,1); //NO!!!
 
         _gamePixmap = new QPixmap(pixmapSize());
         _gamePixmap->fill((QColor)black);
 
-        debug("Resizing blackPixmap to %d---%d",pieceSizeDisp().width(),pieceSizeDisp().height());
-        blackPixmap->resize(pieceSizeDisp());
-        blackPixmap->fill(QColor(black));
+        currentData()->_pieceList->setAutoDelete(true);
 
-	currentData()->_currentPiecePixmap = new QPixmap(pieceSizeDisp());
+        currentData()->_currentPiecePixmap = new QPixmap(pieceSizeDisp());
 	getPiecePixmap();
 
-	// First difficult thing: the game pixmap
         QPixmap piecePxm(currentPiecePixmap()->size());
 
 #define CUR (drawingIterator.current())
@@ -404,9 +375,6 @@ bool KPuzzleApp::initializeLoad()
 	QString s;
 	s.sprintf("%i:%2i",rest / 60,rest % 60);
 	mainWidget()->time()->display(s);
-// 	char timeStr[6];
-// 	snprintf(timeStr,5,"%i:%i",scoreData()->_maxTime / 60,scoreData()->_maxTime % 60);
-// 	mainWidget()->time()->display(timeStr);
 
 	startTimer();
 	setStatus(GS_RUNNING);
@@ -447,92 +415,64 @@ void KPuzzleApp::done()
 KPuzzleApp::~KPuzzleApp()
 {
 	if (status() != GS_NOT_STARTED) done();
+        else debug("KPuzzleApp destructed with status==GS_NOT_STARTED");
 
         delete _scoreData;
         delete _gameData;
         delete _currentData;
-
-        delete blackPixmap;
 }
 
-void KPuzzleApp::slotNewGame()
+bool KPuzzleApp::createNewGame()
 {
-        bool gameIsPaused = paused();
-        pause(true);
-	if (status() != GS_NOT_STARTED)
-		if (KMessageBox::warningContinueCancel(0,i18n("A game is already running. Start a new one and stop the current?"),
-                                                       i18n("Stop this game?"),i18n("Continue")) == KMessageBox::Cancel) {
-                        pause(gameIsPaused);
-                        return;
-                }
-	QString fname = KFileDialog::getOpenFileName(LEVELS_DIR,"*.bmp",mainWidget(),i18n("Open Image"));
+	QString fname = KFileDialog::getOpenFileName(LEVELS_DIR,"*.bmp *.jpg",mainWidget(),i18n("Open Image"));
      	if (fname.isNull()) {
-                pause(gameIsPaused);
-                return;
+                return false;
         }
         CGameDialog gd(mainWidget());
 	if (gd.exec() != QDialog::Accepted) {
-                pause(gameIsPaused);
-                return;
+                return false;
         }
 
         gameData()->_filename = fname;
 
-	if (status() != GS_NOT_STARTED) emit sigTerm(TM_FORCE);
-
  	gameData()->_dialog_pieceSize = gd.pieceSize();
 	gameData()->_gameType = gd.gameType();
 	gameData()->_difficulty = gd.difficulty();
 	gameData()->_scale = true;
 	gameData()->_useMask = gd.useMask();
 
-//         gameData()->_dialog_pieceSize = 3;
-//         gameData()->_gameType = STD_GAME;
-//         gameData()->_difficulty = 1;
-//         gameData()->_scale = true;
-//         gameData()->_useMask = true;
+        _mainPixmap = new QPixmap;
+        _mainPixmap->load(gameData()->_filename);
+        if (_mainPixmap->isNull()) {
+                delete _mainPixmap;
+		return false;
+	}
 
-//        abort();
-
-        mainWidget()->createPlayground(gameData()->_gameType == UNBEARABLE_GAME);
-	initializeNew(); // calls initialize()
+	return true;
 }
 
-void KPuzzleApp::slotOpenGame()
+bool KPuzzleApp::createLoadedGame()
 {
-        bool gameIsPaused = paused();
-        pause(true);
-	if (status() != GS_NOT_STARTED)
-		if (KMessageBox::warningContinueCancel(0,i18n("A game is already running. Stop it and open a saved game?"),
-                                                       i18n("Stop this game?"),i18n("Continue")) == KMessageBox::Cancel) {
-                        pause(gameIsPaused);
-                        return;
-                }
 	gameFilename() = KFileDialog::getOpenFileName(QDir::homeDirPath(),"*.pzz",mainWidget(),i18n("Open Game"));
 	if (gameFilename().isNull()) {
-                pause(gameIsPaused);
-                return;
+                return false;
         }
         if (gameFilename().findRev(".") <= gameFilename().findRev("/")) // There is no dot after the last slash
                 gameFilename() += ".pzz";
         if (!QFile::exists(gameFilename())) {
                 KMessageBox::sorry(0,i18n("The file you specified does not exist"),i18n("File does not exist"));
-                pause(gameIsPaused);
-                return;
+                return false;
         }
         QFile file(gameFilename());
         if (!file.open(IO_ReadOnly)) {
                 KMessageBox::sorry(0,i18n("The file you specified could not be opened for reading. Check the file's permissions."),
                                    i18n("File could not be opened"));
-                pause(gameIsPaused);
-                return;
+                return false;
         }
 
         QDataStream stream(&file);
-//        stream.setPrintableData(true);
-        if (!load(&stream)) return;
-	mainWidget()->createPlayground(gameData()->_gameType == UNBEARABLE_GAME);
-        initializeLoad();
+        if (!load(&stream)) return false;
+        return true;
 }
 
 void KPuzzleApp::slotSaveGame()
@@ -616,16 +556,6 @@ void KPuzzleApp::slotSaveGameAs()
         pause(gameIsPaused);
 }
 
-void KPuzzleApp::slotHighscores()
-{
-        bool gameIsPaused = paused();
-        pause(true);
- 	CHighscoreDialog hs(mainWidget()->parentWidget(),0);
-	hs.initializeDialog();
-	hs.exec();
-        pause(gameIsPaused);
-}
-
 void KPuzzleApp::slotShowLarge()
 {
 	if (status() != GS_RUNNING) return;
@@ -824,17 +754,30 @@ void KPuzzleApp::calcPiecesCount()
 bool KPuzzleApp::scalePixmap()
 {
         gameData()->_originalPixmapSize = pixmapSize();
-	if (pixmapSize().width() % pieceSize().width()) {
+	if (pixmapSize().width() % pieceSize().width() || pixmapSize().height() % pieceSize().height()) {
 		QWMatrix m;
-		float scaleVal = (float) ((pixmapSize().width() / pieceSize().width() + 1) *
+		float scaleValX = (float) ((pixmapSize().width() / pieceSize().width() + 1) *
                         pieceSize().width()) / pixmapSize().width();
-		m.scale(scaleVal,scaleVal);
+		float scaleValY = (float) ((pixmapSize().height() / pieceSize().height() + 1) *
+                        pieceSize().height()) / pixmapSize().height();
+		m.scale(scaleValX,scaleValY);
 		QPixmap* pxm = new QPixmap(_mainPixmap->xForm(m));
 		delete _mainPixmap;
 		_mainPixmap = pxm;
 
                 return true;
 	}
+// 	if (pixmapSize().width() % pieceSize().width()) {
+// 		QWMatrix m;
+// 		float scaleVal = (float) ((pixmapSize().width() / pieceSize().width() + 1) *
+//                         pieceSize().width()) / pixmapSize().width();
+// 		m.scale(scaleVal,scaleVal);
+// 		QPixmap* pxm = new QPixmap(_mainPixmap->xForm(m));
+// 		delete _mainPixmap;
+// 		_mainPixmap = pxm;
+
+//                 return true;
+// 	}
         return false;
 }
 
@@ -1222,14 +1165,19 @@ void KPuzzleApp::setStatus(int s)
 	case GS_NOT_STARTED:
 		enable = MI_NONE;
 
-		done();
+                mainWidget()->showBackground(false);
 
-		break;
+                done();
+                emit sigDone();
+
+		return;
 
 	case GS_RUNNING:
 		enable = MI_STOP_GAME | MI_PAUSE | MI_SHOW_LARGE | MI_SAVE_GAME;
 
-		mainWidget()->freeGamePixmap(GP_BIGLOGO);
+                mainWidget()->showBackground(true);
+
+                mainWidget()->freeGamePixmap(GP_BIGLOGO);
 		mainWidget()->freeGamePixmap(GP_PAUSE);
 
 		break;
@@ -1237,6 +1185,8 @@ void KPuzzleApp::setStatus(int s)
 	case GS_WINNING:
 		enable = MI_STOP_GAME;
 
+                mainWidget()->showBackground(false);
+
 		mainWidget()->showStatusMsg(i18n("Press mouse key to continue"));
 		mainWidget()->showAll(false);
 		mainWidget()->useGamePixmap(GP_WIN);
@@ -1247,6 +1197,8 @@ void KPuzzleApp::setStatus(int s)
 	case GS_SHOW_LARGE:
 		enable = MI_STOP_GAME | MI_SAVE_GAME;
 
+                mainWidget()->showBackground(false);
+
 		mainWidget()->showStatusMsg(i18n("Press mouse key to continue"));
 
 		break;
@@ -1254,6 +1206,8 @@ void KPuzzleApp::setStatus(int s)
 	case GS_SHOW_LARGE_WINNING:
 		enable = MI_STOP_GAME;
 
+                mainWidget()->showBackground(false);
+
 		mainWidget()->showStatusMsg(i18n("Press mouse key to continue"));
 		mainWidget()->setLargePxm(new QPixmap(getLargePixmap(mainWidget()->size())));
 		mainWidget()->freeGamePixmap(GP_WIN);
@@ -1263,6 +1217,8 @@ void KPuzzleApp::setStatus(int s)
 	case GS_LOSING:
 		enable = MI_STOP_GAME;
 
+                mainWidget()->showBackground(false);
+
 		mainWidget()->showStatusMsg(i18n("Press mouse key to continue"));
 		mainWidget()->showAll(false);
 		mainWidget()->useGamePixmap(GP_LOSE);
@@ -1273,6 +1229,8 @@ void KPuzzleApp::setStatus(int s)
 	case GS_SHOW_LARGE_LOSING:
 		enable = MI_STOP_GAME;
 
+                mainWidget()->showBackground(false);
+
 		mainWidget()->showStatusMsg(i18n("Press mouse key to continue"));
 		mainWidget()->setLargePxm(new QPixmap(getLargePixmap(mainWidget()->size())));
 		mainWidget()->freeGamePixmap(GP_LOSE);
@@ -1282,23 +1240,25 @@ void KPuzzleApp::setStatus(int s)
 	case GS_SHOW_HIGHSCORES:
 		enable = MI_NONE;
 
-		showHighscoreDialog();
-		mainWidget()->destroyPlayground();
-		setStatus(GS_NOT_STARTED);
-		mainWidget()->update();
+                mainWidget()->showBackground(false);
 
-		break;
+                showHighscoreDialog();
+		setStatus(GS_NOT_STARTED);
+
+                return;
 
 	case GS_PAUSED:
 		enable = MI_STOP_GAME | MI_SAVE_GAME;
 
+                mainWidget()->showBackground(false);
+
 		mainWidget()->showStatusMsg(i18n("Press mouse key to continue"));
 		mainWidget()->useGamePixmap(GP_PAUSE);
 
 		break;
 	}
         //	((KPuzzle*) (mainWidget()->parentWidget()))->enableMenuItems(enable);
-        enableMenuItems(enable);
+        mainWidget()->enableMenuItems(enable);
 }
 
 void KPuzzleApp::pause(bool p)
@@ -1329,6 +1289,7 @@ void KPuzzleApp::slotTimer()
 	QString s;
 	s.sprintf("%i:%2i",rest / 60,rest % 60);
 	mainWidget()->time()->display(s);
+        if (rest < 0) emit sigTerm(TM_LOST);
 	// Something has to be done, dependent on the game type
 // 	PieceState* current = _pieceList->at(_currentPieceNr);
 // 	switch (_gameType) {
@@ -1367,6 +1328,7 @@ void KPuzzleApp::slotTimer()
 void KPuzzleApp::slotTerm(int reason)
 {
 	stopTimer();
+        currentPieceData()->stopTimer();
 	switch (reason) {
 	case TM_WON:
 		changeScore(SC_COUNT_TIME);
@@ -1392,7 +1354,7 @@ void KPuzzleApp::slotTerm(int reason)
 		case GS_SHOW_LARGE:
 		case GS_SHOW_LARGE_WINNING:
 		case GS_SHOW_LARGE_LOSING:
-			mainWidget()->destroyPlayground();
+//			mainWidget()->destroyPlayground();
 //			setStatus(GS_NOT_STARTED);
 			break;
 		case GS_WINNING:
@@ -1402,21 +1364,8 @@ void KPuzzleApp::slotTerm(int reason)
 		}
 
 		setStatus(GS_NOT_STARTED);
-		mainWidget()->update();
 		return;
 	}
 
 }
 
-void KPuzzleApp::enableMenuItems(int enable)
-{
-        debug("enableMenuItems called!!!");
-        actSave()->setEnabled(enable & MI_SAVE_GAME);
-        actSaveAs()->setEnabled(enable & MI_SAVE_GAME);
-
-        actTerminate()->setEnabled(enable & MI_STOP_GAME);
-        actShowLarge()->setEnabled(enable & MI_SHOW_LARGE);
-        actShowMainPixmap()->setEnabled((enable & MI_SHOW_LARGE) &&
-                                        kapp->sessionConfig()->readBoolEntry("ShowMainPixmap",SV_SHOW_MAIN_PIXMAP));
-        actPause()->setEnabled(enable & MI_SHOW_LARGE);
-}
