@@ -165,7 +165,12 @@ class KPuzzleApp : public QObject
 	bool initializeNew();
 // init method for loaded game
 	bool initializeLoad();
-// called when the game terminates
+
+// called after the game is constructed, calls initialize*
+        bool createNewGame();
+	bool createLoadedGame();
+
+// called before the object is destroyed
 	void done();
 	virtual ~KPuzzleApp();
 
@@ -186,13 +191,13 @@ class KPuzzleApp : public QObject
 
         QString gameFilename() const { return _gameFilename; }
 
-        KAction*& actSave() { return _actSave; }
-	KAction*& actSaveAs()  { return _actSaveAs; }
-	KAction*& actTerminate() { return _actTerminate; }
+//         KAction*& actSave() { return _actSave; }
+// 	KAction*& actSaveAs()  { return _actSaveAs; }
+// 	KAction*& actTerminate() { return _actTerminate; }
 
-	KAction*& actShowLarge() { return _actShowLarge; }
-        KAction*& actShowMainPixmap() { return _actShowMainPixmap; }
-	KAction*& actPause() { return _actPause; }
+// 	KAction*& actShowLarge() { return _actShowLarge; }
+//         KAction*& actShowMainPixmap() { return _actShowMainPixmap; }
+// 	KAction*& actPause() { return _actPause; }
 
 	// Data which does not change during the game
 	ScoreData* scoreData() const { return _scoreData; }
@@ -275,13 +280,13 @@ class KPuzzleApp : public QObject
 
 
 	// Actions for disabling/toggling
-	KAction* _actSave;
-	KAction* _actSaveAs;
-	KAction* _actTerminate;
+// 	KAction* _actSave;
+// 	KAction* _actSaveAs;
+// 	KAction* _actTerminate;
 
-	KAction* _actShowLarge;
-        KAction* _actShowMainPixmap;
-	KAction* _actPause;
+// 	KAction* _actShowLarge;
+//         KAction* _actShowMainPixmap;
+// 	KAction* _actPause;
 
  private:
 
@@ -318,9 +323,6 @@ class KPuzzleApp : public QObject
 
         void stopGame(int why) { emit sigTerm(why); }
 
-// Changes menu items, according to the status and to configuration settings
-	void enableMenuItems(int enable);
-
  protected:
 // Writes pixmap of piece pc or current pixmap to pxm of _currentData->_currentPiecePixmap
 	void getPiecePixmap(CPiece* pc = NULL,QPixmap* pxm = NULL) const;
@@ -336,12 +338,10 @@ class KPuzzleApp : public QObject
         bool load(QDataStream* f);
 
  public slots:
-	void slotNewGame();
-	void slotOpenGame();
 	void slotSaveGame();
 	void slotSaveGameAs();
         void slotStopGame() { stopGame(TM_QUIT); }
-        void slotHighscores();
+//        void slotHighscores();
 	void slotPause();
 	void slotPauseToggle(bool on);
         void slotShowLarge();
@@ -356,6 +356,8 @@ class KPuzzleApp : public QObject
 	void sigPiecePlaced();
 	void sigWinning();
 	void sigTerm(int);
+        void sigDone(); // signalizes that this game object has done what it should do
+                        // and should be destroyed
 	void sigScore(int);
 //	void sigTime(const char*);
 	void sigUpdate();
