@@ -96,9 +96,11 @@ QPoint CPicview::DPtoLP(QPoint p)
 
 void CPicview::updatePixmap(QPixmap* p,QRect udRect)
 {
+        ASSERT(PICVIEW_WIDTH == this->width());
+        ASSERT(PICVIEW_HEIGHT == this->height());
 	if (p) {
 		QWMatrix sc;
-                if (p->height() / p->width() > 0.75)
+                if ((float) p->height() / p->width() > 0.75)
                         sc.scale(PICVIEW_HEIGHT / p->height(),
                                  PICVIEW_HEIGHT / p->height());
                 else
